package ir.metrix.flutter

import androidx.annotation.NonNull

import android.net.Uri

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.MethodCall

import ir.metrix.messaging.RevenueCurrency
import ir.metrix.OnDeeplinkResponseListener

import java.util.HashMap

/**
 * MetrixSDKFlutterPlugin
 */
class Metrix : FlutterPlugin, MethodChannel.MethodCallHandler {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channel: MethodChannel

    private lateinit var binaryMessengerInstance: BinaryMessenger

    private val sessionIdEvent: EventChannel by lazy { EventChannel(binaryMessengerInstance, "MetrixSessionIdEvent") }
    private val sessionNumEvent: EventChannel by lazy { EventChannel(binaryMessengerInstance, "MetrixSessionNumEvent") }
    private val userIdEvent: EventChannel by lazy { EventChannel(binaryMessengerInstance, "MetrixUserIdEvent") }
    private val attributionInfoEvent: EventChannel by lazy { EventChannel(binaryMessengerInstance, "MetrixAttributionEvent") }

    private val sessionIdReader: SessionIdReader by lazy { SessionIdReader() }
    private val sessionNumReader: SessionNumReader by lazy { SessionNumReader() }
    private val userIdReader: UserIdReader by lazy { UserIdReader() }
    private val attributionInfoStreamHandler: AttributionInfoStreamHandler by lazy { AttributionInfoStreamHandler() }

    
    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        binaryMessengerInstance = flutterPluginBinding.binaryMessenger
        channel = MethodChannel(binaryMessengerInstance, "Metrix")
        channel.setMethodCallHandler(this)
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: MethodChannel.Result) {
        when (call.method) {
            "newEvent" -> {
                val slug = call.argument<String>("slug") ?: ""
                val attr = call.argument<Map<String, String>>("attributes") ?: mapOf()
                ir.metrix.Metrix.newEvent(slug, attr)
            }
            "addUserAttributes" -> {
                val attr = call.argument<Map<String, String>>("attributes") ?: mapOf()
                ir.metrix.Metrix.addUserAttributes(attr)
            }
            "initSessionIdListener" -> {
                sessionIdEvent.setStreamHandler(sessionIdReader)
            }
            "initSessionNumListener" -> {
                sessionNumEvent.setStreamHandler(sessionNumReader)
            }
            "initUserIdListener" -> {
                userIdEvent.setStreamHandler(userIdReader)
            }
            "newRevenue" -> {
                val slug = call.argument<String>("slug") ?: ""
                val amount = call.argument<Double>("amount") ?: 0.0
                val currency = call.argument<Int>("currency")
                val orderId = call.argument<String>("orderId")
                var revenueCurrency = RevenueCurrency.IRR
                if (currency != null && currency == 1) revenueCurrency = RevenueCurrency.USD
                if (currency != null && currency == 2) revenueCurrency = RevenueCurrency.EUR
                ir.metrix.Metrix.newRevenue(slug, amount, revenueCurrency, orderId)
            }
            "setPushToken" -> {
                val token = call.argument<String>("token")
                if (token != null) {
                    ir.metrix.Metrix.setPushToken(token)
                }
            }
            "initAttributionListener" -> {
                attributionInfoEvent.setStreamHandler(attributionInfoStreamHandler)
            }
            "getDeeplinkResponse" -> {
                val shouldLaunchDeeplink = call.argument<Boolean>("shouldLaunchDeeplink") ?: false
                ir.metrix.Metrix.setOnDeeplinkResponseListener(
                    object : OnDeeplinkResponseListener {
                        override fun launchReceivedDeeplink(deeplink: Uri): Boolean {
                            result.success(deeplink.toString())
                            return shouldLaunchDeeplink
                        }
                    })
            }
            else -> result.notImplemented()
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }
}